module xillydemo
  (
  inout [33:0] J3, J8,
  inout cape_button,

  output  hdmi_clk_p,
  output  hdmi_clk_n,
  output [2:0] hdmi_d_p,
  output [2:0] hdmi_d_n,
  output  hdmi_out_en
  );

   wire       ap_clk;
   wire       quiesce;

   // Memory arrays

   reg [7:0] litearray0[0:31];
   reg [7:0] litearray1[0:31];
   reg [7:0] litearray2[0:31];
   reg [7:0] litearray3[0:31];

   // Wires related to Xillybus Lite
   wire        user_clk;
   wire        user_wren;
   wire [3:0]  user_wstrb;
   wire        user_rden;
   reg [31:0]  user_rd_data;
   wire [31:0] user_wr_data;
   wire [31:0] user_addr;
   wire        user_irq;

   wire [63:0] gpio_tri_i, gpio_tri_o, gpio_tri_t;
   genvar      i;

   generate
      for (i=0; i<34; i=i+1)
	begin: gpio
	   assign gpio_tri_i[i] = J8[i];
	   assign J8[i] = gpio_tri_t[i] ? 1'bz : gpio_tri_o[i];
	end
   endgenerate

   assign gpio_tri_i[34] = cape_button;
   assign cape_button = gpio_tri_t[34] ? 1'bz : gpio_tri_o[34];

   assign hdmi_out_en = 1;
   assign J3 = 0;

   vivado_system vivado_system_ins
     (
      .ap_clk(ap_clk),
      .quiesce(quiesce),

      .GPIO_0_tri_i(gpio_tri_i),
      .GPIO_0_tri_o(gpio_tri_o),
      .GPIO_0_tri_t(gpio_tri_t),

      .user_addr(user_addr),
      .user_clk(user_clk),
      .user_irq(user_irq),
      .user_rd_data(user_rd_data),
      .user_rden(user_rden),
      .user_wr_data(user_wr_data),
      .user_wren(user_wren),
      .user_wstrb(user_wstrb),

      .dvi_clk_n(hdmi_clk_n),
      .dvi_clk_p(hdmi_clk_p),
      .dvi_d_n(hdmi_d_n),
      .dvi_d_p(hdmi_d_p)
      );

   assign      user_irq = 0; // No interrupts for now

   always @(posedge user_clk)
     begin
	if (user_wstrb[0])
	  litearray0[user_addr[6:2]] <= user_wr_data[7:0];

	if (user_wstrb[1])
	  litearray1[user_addr[6:2]] <= user_wr_data[15:8];

	if (user_wstrb[2])
	  litearray2[user_addr[6:2]] <= user_wr_data[23:16];

	if (user_wstrb[3])
	  litearray3[user_addr[6:2]] <= user_wr_data[31:24];

	if (user_rden)
	  user_rd_data <= { litearray3[user_addr[6:2]],
			    litearray2[user_addr[6:2]],
			    litearray1[user_addr[6:2]],
			    litearray0[user_addr[6:2]] };
     end
endmodule
